@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright 2003,2004,2005 Free Software Foundation, Inc.
+# Copyright 2003,2004,2005,2006 Free Software Foundation, Inc.
 # 
 # This file is part of GNU Radio
 # 
@@ -20,7 +20,7 @@
 # Boston, MA 02111-1307, USA.
 # 
 
-from gnuradio import gr, gru
+from gnuradio import gr, gru, window
 from gnuradio.wxgui import stdgui
 import wx
 import gnuradio.wxgui.plot as plot
@@ -29,10 +29,12 @@ import threading
 import math    
 
 default_fftsink_size = (640,240)
+default_fft_rate = gr.prefs().get_long('wxgui', 'fft_rate', 15)
 
 class fft_sink_base(object):
     def __init__(self, input_is_real=False, baseband_freq=0, y_per_div=10, ref_level=50,
-                 sample_rate=1, fft_size=512, fft_rate=15,
+                 sample_rate=1, fft_size=512,
+                 fft_rate=default_fft_rate,
                  average=False, avg_alpha=None, title='', peak_hold=False):
 
         # initialize common attributes
@@ -83,8 +85,8 @@ class fft_sink_base(object):
 class fft_sink_f(gr.hier_block, fft_sink_base):
     def __init__(self, fg, parent, baseband_freq=0,
                  y_per_div=10, ref_level=50, sample_rate=1, fft_size=512,
-                 fft_rate=15, average=False, avg_alpha=None, title='',
-                 size=default_fftsink_size, peak_hold=False):
+                 fft_rate=default_fft_rate, average=False, avg_alpha=None,
+                 title='', size=default_fftsink_size, peak_hold=False):
 
         fft_sink_base.__init__(self, input_is_real=True, baseband_freq=baseband_freq,
                                y_per_div=y_per_div, ref_level=ref_level,
@@ -93,13 +95,22 @@ class fft_sink_f(gr.hier_block, fft_sink_base):
                                average=average, avg_alpha=avg_alpha, title=title,
                                peak_hold=peak_hold)
                                
-        s2p = gr.serial_to_parallel(gr.sizeof_float, fft_size)
+        s2p = gr.stream_to_vector(gr.sizeof_float, fft_size)
         one_in_n = gr.keep_one_in_n(gr.sizeof_float * fft_size,
                                     max(1, int(sample_rate/fft_size/fft_rate)))
-        fft = gr.fft_vfc(fft_size, True, True)
+
+        mywindow = window.blackmanharris(fft_size)
+        fft = gr.fft_vfc(fft_size, True, mywindow)
+        power = 0
+        for tap in mywindow:
+            power += tap*tap
+            
         c2mag = gr.complex_to_mag(fft_size)
         self.avg = gr.single_pole_iir_filter_ff(1.0, fft_size)
-        log = gr.nlog10_ff(20, fft_size, -20*math.log10(fft_size))
+
+        # FIXME  We need to add 3dB to all bins but the DC bin
+        log = gr.nlog10_ff(20, fft_size,
+                           -20*math.log10(fft_size)-10*math.log10(power/fft_size))
         sink = gr.message_sink(gr.sizeof_float * fft_size, self.msgq, True)
 
         fg.connect (s2p, one_in_n, fft, c2mag, self.avg, log, sink)
@@ -112,8 +123,8 @@ class fft_sink_f(gr.hier_block, fft_sink_base):
 class fft_sink_c(gr.hier_block, fft_sink_base):
     def __init__(self, fg, parent, baseband_freq=0,
                  y_per_div=10, ref_level=50, sample_rate=1, fft_size=512,
-                 fft_rate=15, average=False, avg_alpha=None, title='',
-                 size=default_fftsink_size, peak_hold=False):
+                 fft_rate=default_fft_rate, average=False, avg_alpha=None,
+                 title='', size=default_fftsink_size, peak_hold=False):
 
         fft_sink_base.__init__(self, input_is_real=False, baseband_freq=baseband_freq,
                                y_per_div=y_per_div, ref_level=ref_level,
@@ -122,13 +133,18 @@ class fft_sink_c(gr.hier_block, fft_sink_base):
                                average=average, avg_alpha=avg_alpha, title=title,
                                peak_hold=peak_hold)
 
-        s2p = gr.serial_to_parallel(gr.sizeof_gr_complex, fft_size)
+        s2p = gr.stream_to_vector(gr.sizeof_gr_complex, fft_size)
         one_in_n = gr.keep_one_in_n(gr.sizeof_gr_complex * fft_size,
                                     max(1, int(sample_rate/fft_size/fft_rate)))
-        fft = gr.fft_vcc(fft_size, True, True)
+        mywindow = window.blackmanharris(fft_size)
+        power = 0
+        for tap in mywindow:
+            power += tap*tap
+            
+        fft = gr.fft_vcc(fft_size, True, mywindow)
         c2mag = gr.complex_to_mag(fft_size)
         self.avg = gr.single_pole_iir_filter_ff(1.0, fft_size)
-        log = gr.nlog10_ff(20, fft_size, -20*math.log10(fft_size))
+        log = gr.nlog10_ff(20, fft_size, -20*math.log10(fft_size)-10*math.log10(power/fft_size))
         sink = gr.message_sink(gr.sizeof_float * fft_size, self.msgq, True)
 
         fg.connect(s2p, one_in_n, fft, c2mag, self.avg, log, sink)
@@ -430,11 +446,11 @@ class test_app_flow_graph (stdgui.gui_flow_graph):
         fft_size = 256
 
         # build our flow graph
-        input_rate = 20.000e3
+        input_rate = 20.48e3
 
         # Generate a complex sinusoid
-        src1 = gr.sig_source_c (input_rate, gr.GR_SIN_WAVE, 5.75e3, 1000)
-        #src1 = gr.sig_source_c (input_rate, gr.GR_CONST_WAVE, 5.75e3, 1000)
+        #src1 = gr.sig_source_c (input_rate, gr.GR_SIN_WAVE, 2e3, 1)
+        src1 = gr.sig_source_c (input_rate, gr.GR_CONST_WAVE, 5.75e3, 1)
 
         # We add these throttle blocks so that this demo doesn't
         # suck down all the CPU available.  Normally you wouldn't use these.
@@ -442,16 +458,16 @@ class test_app_flow_graph (stdgui.gui_flow_graph):
 
         sink1 = fft_sink_c (self, panel, title="Complex Data", fft_size=fft_size,
                             sample_rate=input_rate, baseband_freq=100e3,
-                            ref_level=60, y_per_div=10)
+                            ref_level=0, y_per_div=20)
         vbox.Add (sink1.win, 1, wx.EXPAND)
         self.connect (src1, thr1, sink1)
 
-        src2 = gr.sig_source_f (input_rate, gr.GR_SIN_WAVE, 5.75e3, 1000)
-        #src2 = gr.sig_source_f (input_rate, gr.GR_CONST_WAVE, 5.75e3, 1000)
+        #src2 = gr.sig_source_f (input_rate, gr.GR_SIN_WAVE, 2e3, 1)
+        src2 = gr.sig_source_f (input_rate, gr.GR_CONST_WAVE, 5.75e3, 1)
         thr2 = gr.throttle(gr.sizeof_float, input_rate)
         sink2 = fft_sink_f (self, panel, title="Real Data", fft_size=fft_size*2,
                             sample_rate=input_rate, baseband_freq=100e3,
-                            ref_level=60, y_per_div=10)
+                            ref_level=0, y_per_div=20)
         vbox.Add (sink2.win, 1, wx.EXPAND)
         self.connect (src2, thr2, sink2)
 
