@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright 2003,2004 Free Software Foundation, Inc.
+# Copyright 2003,2004,2006 Free Software Foundation, Inc.
 # 
 # This file is part of GNU Radio
 # 
@@ -29,25 +29,31 @@ import threading
 import struct
 
 default_scopesink_size = (640, 240)
+default_v_scale = 1000
+default_frame_decim = gr.prefs().get_long('wxgui', 'frame_decim', 1)
 
 class scope_sink_f(gr.hier_block):
     def __init__(self, fg, parent, title='', sample_rate=1,
-                 size=default_scopesink_size):
+                 size=default_scopesink_size, frame_decim=default_frame_decim,
+                 v_scale=default_v_scale, t_scale=None):
         msgq = gr.msg_queue(2)         # message queue that holds at most 2 messages
-        guts = gr.oscope_sink_f (sample_rate, msgq)
+        guts = gr.oscope_sink_f(sample_rate, msgq)
         gr.hier_block.__init__(self, fg, guts, guts)
-        self.win = scope_window (win_info (msgq, sample_rate, guts, title), parent)
+        self.win = scope_window(win_info (msgq, sample_rate, frame_decim,
+                                          v_scale, t_scale, guts, title), parent)
 
 class scope_sink_c(gr.hier_block):
     def __init__(self, fg, parent, title='', sample_rate=1,
-                 size=default_scopesink_size):
+                 size=default_scopesink_size, frame_decim=default_frame_decim,
+                 v_scale=default_v_scale, t_scale=None):
         msgq = gr.msg_queue(2)         # message queue that holds at most 2 messages
         c2f = gr.complex_to_float()
-        guts = gr.oscope_sink_f (sample_rate, msgq)
+        guts = gr.oscope_sink_f(sample_rate, msgq)
         fg.connect((c2f, 0), (guts, 0))
         fg.connect((c2f, 1), (guts, 1))
         gr.hier_block.__init__(self, fg, c2f, guts)
-        self.win = scope_window (win_info (msgq, sample_rate, guts, title), parent)
+        self.win = scope_window(win_info(msgq, sample_rate, frame_decim,
+                                         v_scale, t_scale, guts, title), parent)
         
 
 # ========================================================================
@@ -85,6 +91,31 @@ time_base_list = [                      # time / division
     5.0e-2
     ]
 
+v_scale_list = [ # counts / div, LARGER gains are SMALLER /div, appear EARLIER
+    2.0e-3,   # 2m / div, don't call it V/div it's actually counts/div
+    5.0e-3,
+    1.0e-2,
+    2.0e-2,
+    5.0e-2,
+    1.0e-1,
+    2.0e-1,
+    5.0e-1,
+    1.0e+0,
+    2.0e+0,
+    5.0e+0,
+    1.0e+1,
+    2.0e+1,
+    5.0e+1,
+    1.0e+2,
+    2.0e+2,
+    5.0e+2,
+    1.0e+3,
+    2.0e+3,
+    5.0e+3,
+    1.0e+4 # 10000 /div, USRP full scale is -/+ 32767
+    ]
+
+    
 wxDATA_EVENT = wx.NewEventType()
 
 def EVT_DATA_EVENT(win, func):
@@ -101,23 +132,36 @@ class DataEvent(wx.PyEvent):
 
 
 class win_info (object):
-    __slots__ = ['msgq', 'sample_rate', 'scopesink', 'title',
-                 'time_scale_cursor', 'marker', 'xy', 'running']
+    __slots__ = ['msgq', 'sample_rate', 'frame_decim', 'v_scale', 
+                 'scopesink', 'title',
+                 'time_scale_cursor', 'v_scale_cursor', 'marker', 'xy',
+                 'autorange', 'running']
 
-    def __init__ (self, msgq, sample_rate, scopesink, title = "Oscilloscope"):
+    def __init__ (self, msgq, sample_rate, frame_decim, v_scale, t_scale,
+                  scopesink, title = "Oscilloscope"):
         self.msgq = msgq
         self.sample_rate = sample_rate
+        self.frame_decim = frame_decim
         self.scopesink = scopesink
         self.title = title;
-        self.time_scale_cursor = gru.seq_with_cursor (time_base_list)
-        self.time_scale_cursor.prev ()
+
+        self.time_scale_cursor = gru.seq_with_cursor(time_base_list, initial_value = t_scale)
+        self.v_scale_cursor = gru.seq_with_cursor(v_scale_list, initial_value = v_scale)
+
         self.marker = 'line'
         self.xy = False
+        if v_scale == None:        # 0 and None are both False, but 0 != None
+            self.autorange = True
+        else:
+            self.autorange = False # 0 is a valid v_scale            
         self.running = True
 
     def get_time_per_div (self):
         return self.time_scale_cursor.current ()
 
+    def get_volts_per_div (self):
+        return self.v_scale_cursor.current ()
+
     def get_sample_rate (self):
         return self.sample_rate
 
@@ -130,13 +174,15 @@ class win_info (object):
     def get_marker (self):
         return self.marker
 
-        
+
 class input_watcher (threading.Thread):
-    def __init__ (self, msgq, event_receiver, **kwds):
+    def __init__ (self, msgq, event_receiver, frame_decim, **kwds):
         threading.Thread.__init__ (self, **kwds)
         self.setDaemon (1)
         self.msgq = msgq
         self.event_receiver = event_receiver
+        self.frame_decim = frame_decim
+        self.iscan = 0
         self.keep_running = True
         self.start ()
 
@@ -144,27 +190,33 @@ class input_watcher (threading.Thread):
         # print "input_watcher: pid = ", os.getpid ()
         while (self.keep_running):
             msg = self.msgq.delete_head()   # blocking read of message queue
-            nchan = int(msg.arg1())         # number of channels of data in msg
-            nsamples = int(msg.arg2())      # number of samples in each channel
+            if self.iscan == 0:            # only display at frame_decim
+                self.iscan = self.frame_decim
+                                
+                nchan = int(msg.arg1())    # number of channels of data in msg
+                nsamples = int(msg.arg2()) # number of samples in each channel
 
-            s = msg.to_string()             # get the body of the msg as a string
-            
-            bytes_per_chan = nsamples * gr.sizeof_float
+                s = msg.to_string()      # get the body of the msg as a string
 
-            records = []
-            for ch in range (nchan):
+                bytes_per_chan = nsamples * gr.sizeof_float
 
-                start = ch * bytes_per_chan
-                chan_data = s[start:start+bytes_per_chan]
-                rec = Numeric.fromstring (chan_data, Numeric.Float32)
-                records.append (rec)
+                records = []
+                for ch in range (nchan):
 
-            # print "nrecords = %d, reclen = %d" % (len (records), nsamples)
-            
-            de = DataEvent (records)
-            wx.PostEvent (self.event_receiver, de)
-            records = []
-            del de
+                    start = ch * bytes_per_chan
+                    chan_data = s[start:start+bytes_per_chan]
+                    rec = Numeric.fromstring (chan_data, Numeric.Float32)
+                    records.append (rec)
+
+                # print "nrecords = %d, reclen = %d" % (len (records),nsamples)
+
+                de = DataEvent (records)
+                wx.PostEvent (self.event_receiver, de)
+                records = []
+                del de
+
+            # end if iscan == 0
+            self.iscan -= 1
     
 
 class scope_window (wx.Panel):
@@ -179,14 +231,44 @@ class scope_window (wx.Panel):
         self.graph = graph_window (info, self, -1)
 
         vbox.Add (self.graph, 1, wx.EXPAND)
-        vbox.Add (self.make_control_box (), 0, wx.EXPAND)
+        vbox.Add (self.make_control_box(), 0, wx.EXPAND)
+        vbox.Add (self.make_control2_box(), 0, wx.EXPAND)
 
         self.sizer = vbox
         self.SetSizer (self.sizer)
         self.SetAutoLayout (True)
         self.sizer.Fit (self)
+        self.set_autorange(self.info.autorange)
         
 
+    # second row of control buttons etc. appears BELOW control_box
+    def make_control2_box (self):
+        ctrlbox = wx.BoxSizer (wx.HORIZONTAL)
+
+        self.inc_v_button = wx.Button (self, 1101, " < ", style=wx.BU_EXACTFIT)
+        self.inc_v_button.SetToolTipString ("Increase vertical range")
+        wx.EVT_BUTTON (self, 1101, self.incr_v_scale) # ID matches button ID above
+
+        self.dec_v_button  = wx.Button (self, 1100, " > ", style=wx.BU_EXACTFIT)
+        self.dec_v_button.SetToolTipString ("Decrease vertical range")
+        wx.EVT_BUTTON (self, 1100, self.decr_v_scale)
+
+        self.v_scale_label = wx.StaticText (self, 1002, "None") # vertical /div
+        self.update_v_scale_label ()
+
+        self.autorange_checkbox = wx.CheckBox (self, 1102, "Autorange")
+        self.autorange_checkbox.SetToolTipString ("Select autorange on/off")
+        wx.EVT_CHECKBOX(self, 1102, self.autorange_checkbox_event)
+
+        ctrlbox.Add ((5,0) ,0) # left margin space
+        ctrlbox.Add (self.inc_v_button, 0, wx.EXPAND)
+        ctrlbox.Add (self.dec_v_button, 0, wx.EXPAND)
+        ctrlbox.Add (self.v_scale_label, 0, wx.ALIGN_CENTER)
+        ctrlbox.Add ((20,0) ,0) # spacer
+        ctrlbox.Add (self.autorange_checkbox, 0, wx.ALIGN_CENTER)
+
+        return ctrlbox
+
     def make_control_box (self):
         ctrlbox = wx.BoxSizer (wx.HORIZONTAL)
 
@@ -262,11 +344,47 @@ class scope_window (wx.Panel):
     def incr_timebase (self, evt):
         self.info.time_scale_cursor.next ()
         self.update_timebase_label ()
+
+    def update_v_scale_label (self):
+        volts_per_div = self.info.get_volts_per_div ()
+        s = ' ' + eng_notation.num_to_str (volts_per_div) + '/div' # Not V/div
+        self.v_scale_label.SetLabel (s)
+        
+    def decr_v_scale (self, evt):
+        self.info.v_scale_cursor.prev ()
+        self.update_v_scale_label ()
+
+    def incr_v_scale (self, evt):
+        self.info.v_scale_cursor.next ()
+        self.update_v_scale_label ()
         
     def marker_choice_event (self, evt):
         s = evt.GetString ()
         self.set_marker (s)
 
+    def set_autorange(self, on):
+        if on:
+            self.v_scale_label.SetLabel(" (auto)")
+            self.info.autorange = True
+            self.autorange_checkbox.SetValue(True)
+            self.inc_v_button.Enable(False)
+            self.dec_v_button.Enable(False)
+        else:
+            if self.graph.y_range:
+                (l,u) = self.graph.y_range # found by autorange
+                self.info.v_scale_cursor.set_index_by_value((u-l)/8.0)
+            self.update_v_scale_label()
+            self.info.autorange = False
+            self.autorange_checkbox.SetValue(False)
+            self.inc_v_button.Enable(True)
+            self.dec_v_button.Enable(True)
+            
+    def autorange_checkbox_event(self, evt):
+        if evt.Checked():
+            self.set_autorange(True)
+        else:
+            self.set_autorange(False)
+            
     def set_marker (self, s):
         self.info.set_marker (s)        # set info for drawing routines
         i = self.marker_choice.FindString (s)
@@ -336,11 +454,11 @@ class graph_window (plot.PlotCanvas):
 
         EVT_DATA_EVENT (self, self.format_data)
 
-        self.input_watcher = input_watcher (info.msgq, self)
+        self.input_watcher = input_watcher (info.msgq, self, info.frame_decim)
 
     def channel_color (self, ch):
         return self.channel_colors[ch % len(self.channel_colors)]
-        
+       
     def format_data (self, evt):
         if not self.info.running:
             return
@@ -395,9 +513,12 @@ class graph_window (plot.PlotCanvas):
                                       xLabel = '', yLabel = '')
 
         time_per_div = info.get_time_per_div ()
-        x_range = (-5.0 * time_per_div, 5.0 * time_per_div)
+        x_range = (-5.0 * time_per_div, 5.0 * time_per_div) # ranges are tuples!
+        volts_per_div = info.get_volts_per_div ()
+        if not self.info.autorange:
+            self.y_range = (-4.0 * volts_per_div, 4.0 * volts_per_div)
         self.Draw (graphics, xAxis=x_range, yAxis=self.y_range)
-        self.update_y_range ()
+        self.update_y_range () # autorange to self.y_range
 
 
     def format_xy_data (self, evt):
@@ -440,14 +561,16 @@ class graph_window (plot.PlotCanvas):
         graphics = self.last_draw[0]
         p1, p2 = graphics.boundingBox ()     # min, max points of graphics
 
-        if self.avg_y_min:
+        if self.avg_y_min: # prevent vertical scale from jumping abruptly --?
             self.avg_y_min = p1[1] * alpha + self.avg_y_min * (1 - alpha)
             self.avg_y_max = p2[1] * alpha + self.avg_y_max * (1 - alpha)
-        else:
-            self.avg_y_min = p1[1]
-            self.avg_y_max = p2[1]
+        else: # initial guess
+            self.avg_y_min = p1[1] # -500.0 workaround, sometimes p1 is ~ 10^35
+            self.avg_y_max = p2[1] # 500.0
 
         self.y_range = self._axisInterval ('auto', self.avg_y_min, self.avg_y_max)
+        # print "p1 %s  p2 %s  y_min %s  y_max %s  y_range %s" \
+        #        % (p1, p2, self.avg_y_min, self.avg_y_max, self.y_range)
 
 
     def update_x_range (self):
@@ -473,6 +596,23 @@ class test_app_flow_graph (stdgui.gui_flow_graph):
     def __init__(self, frame, panel, vbox, argv):
         stdgui.gui_flow_graph.__init__ (self, frame, panel, vbox, argv)
 
+        if len(argv) > 1:
+            frame_decim = int(argv[1]) 
+        else:
+            frame_decim = 1
+
+        if len(argv) > 2:
+            v_scale = float(argv[2])  # start up at this v_scale value
+        else:
+            v_scale = None  # start up in autorange mode, default
+
+        if len(argv) > 3:
+            t_scale = float(argv[3])  # start up at this t_scale value
+        else:
+            t_scale = None  # old behavior
+
+        print "frame decim %s  v_scale %s  t_scale %s" % (frame_decim,v_scale,t_scale)
+            
         input_rate = 1e6
 
         # Generate a complex sinusoid
@@ -482,7 +622,9 @@ class test_app_flow_graph (stdgui.gui_flow_graph):
         # all the CPU available.  You normally wouldn't use it...
         throttle = gr.throttle(gr.sizeof_gr_complex, input_rate)
 
-        scope = scope_sink_c (self, panel, "Secret Data", sample_rate=input_rate)
+        scope = scope_sink_c (self, panel,"Secret Data",sample_rate=input_rate,
+                              frame_decim=frame_decim,
+                              v_scale=v_scale, t_scale=t_scale)
         vbox.Add (scope.win, 1, wx.EXPAND)
 
         # wire the blocks together
